class MapHandler : EventHandler
{
	ui bool mapActive;
    int playerCount;
    int respawnNum;
    int stepTics;

    override void WorldLoaded(WorldEvent e)
    {
        for (uint i = 0; i < Level.Sides.Size(); i++)
        {
            Side side = Level.Sides[i];
            
            if (side.GetTexture(1) == TexMan.CheckForTexture("KPTX119q", TexMan.TYPE_ANY)
            || side.GetTexture(1) == TexMan.CheckForTexture("KPTX1194", TexMan.TYPE_ANY)
            || side.GetTexture(1) == TexMan.CheckForTexture("KPTX1345", TexMan.TYPE_ANY)
            || side.GetTexture(1) == TexMan.CheckForTexture("M_TREGR", TexMan.TYPE_ANY)
            || side.GetTexture(1) == TexMan.CheckForTexture("GRSSF0", TexMan.TYPE_ANY)
            || side.GetTexture(1) == TexMan.CheckForTexture("PLNTC0", TexMan.TYPE_ANY)
            )
            {
                side.linedef.activation |= SPAC_CROSS | SPAC_MCROSS;
                side.linedef.special = 1000;
                side.linedef.flags |= 0x00000200; //ML_REPEAT_SPECIAL
            }
        }
    }

    override void WorldThingSpawned(WorldEvent e)
    {
        if (multiplayer)
        {
            if (e.thing.bIsMonster)
            {
                e.thing.health *= playerCount * 0.65;
                e.thing.speed *= min(2, playerCount * 0.5);
                e.thing.painchance /= playerCount * 0.5;
            }
            if ((e.thing is "Ammo" || e.thing is "Health" || e.thing is "Armor")
            && !Inventory(e.thing).bTOSSED
            && e.thing.stamina != 100)
            {
                for (int i = 1; i < min(3, playerCount); i++)
                { 
                    let item = Actor.Spawn(e.thing.GetClassName(), e.thing.pos + (Random[Item](-e.thing.radius, e.thing.radius), Random[Item](-e.thing.radius, e.thing.radius), 0));
                    item.stamina = 100;
                    Inventory(item).amount *= 0.25;
                }
            }
        }
    }
	
	override void WorldTick()
	{
		if (stepTics > 0)
            stepTics--;
        
        if (stepTics <= 0)
        {
            if (players[consoleplayer].onground)
            {
                let plr = players[consoleplayer].mo;
                
                if (plr.floorpic == TexMan.CheckForTexture("BONEZNE1", TexMan.TYPE_ANY))
                {
                    plr.A_StartSound("Crunch", CHAN_BODY, CHANF_OVERLAP, volume: plr.vel.xy.Length() * 0.1);
                    stepTics = 15;
                }
				else if (plr.floorpic == TexMan.CheckForTexture("MUD1_01", TexMan.TYPE_ANY)
				|| plr.floorpic == TexMan.CheckForTexture("MUD2_01", TexMan.TYPE_ANY)
				)
				{
                    plr.A_StartSound("Wade", CHAN_BODY, CHANF_OVERLAP, volume: plr.vel.xy.Length() * 0.1);
                    stepTics = 15;
                }
            }
        }
	}

    int respawnLines[100000];

    override void WorldLinePreActivated(WorldEvent e)
    {
        if (e.ActivatedLine.special == 1000)
        {
            e.thing.A_StartSound("Bush", CHAN_BODY, CHANF_OVERLAP);
        }
        else if (e.ActivatedLine.special == 1001)
        {
            if (!respawnLines[e.ActivatedLine.Index()])
                respawnNum++;
            
            respawnLines[e.ActivatedLine.Index()] = 1;
        }
    }

    override void PlayerEntered(PlayerEvent e)
    {
        playerCount++;
    }

    override void PlayerDisconnected(PlayerEvent e)
    {
        playerCount--;
    }

    override void PlayerRespawned(PlayerEvent e)
    {
        if (Level.LevelName == "CountryCide")
        {
            switch (respawnNum)
            {
                case 1:
                    players[e.PlayerNumber].mo.SetOrigin((12624, 752, -64), false);
                    break;
                
                case 2:
                    players[e.PlayerNumber].mo.SetOrigin((11840, 4096, -366), false);
                    break;
                
                case 3:
                    players[e.PlayerNumber].mo.SetOrigin((1280, -256, -360), false);
                    break;
                
                case 4:
                    players[e.PlayerNumber].mo.SetOrigin((10680, 7248, -82), false);
                    break;
                
                case 5:
                    players[e.PlayerNumber].mo.SetOrigin((11000, 6648, 53), false);
                    break;
                
                case 6:
                    players[e.PlayerNumber].mo.SetOrigin((704, 4776, -240), false);
                    break;
                
                case 7:
                    players[e.PlayerNumber].mo.SetOrigin((-2192, 5656, -440), false);
                    break;
                
                case 8:
                    players[e.PlayerNumber].mo.SetOrigin((6496, 7040, -112), false);
                    break;
                
                case 9:
                    players[e.PlayerNumber].mo.SetOrigin((6976, 10064, -96), false);
                    break;
                
                case 10:
                    players[e.PlayerNumber].mo.SetOrigin((11000, 6648, 53), false);
                    break;
                
            }
        }
    }
	
	override bool InputProcess(InputEvent e)
	{
		int in1, in2;
		[in1, in2] = Bindings.GetKeysForCommand("togglemap");
		if ((in1 && in1 == e.KeyScan) || (in2 && in2 == e.KeyScan))
		{
			if (e.Type == InputEvent.Type_KeyDown)
				mapActive = !mapActive;
			
			return true;
		}
		
		return false;
	}

    override void NetworkProcess(ConsoleEvent e)
    {
        if (e.Name == "DropAmmo")
        {
            PlayerInfo plr = players[e.Player];

            if (plr.readyWeapon && plr.readyWeapon is "TFWeaponBase")
            {
                Ammo type = plr.readyWeapon.Ammo1;
                let typeReserve = TFWeaponBase(plr.readyWeapon).AmmoReserveType;

                if (!type || !typeReserve) return;

                int amount = Clamp(plr.mo.CountInv(typeReserve), 0, type.MaxAmount);

                if (amount)
                {
                    plr.mo.TakeInventory(typeReserve, amount);
                    let dropped = plr.mo.Spawn(typeReserve, plr.mo.pos + (64 * cos(plr.mo.angle), 64 * sin(plr.mo.angle), 0));
                    Ammo(dropped).bTOSSED = true;
                    Ammo(dropped).Amount = amount;
                    dropped.Vel3DFromAngle(5, plr.mo.angle, plr.mo.pitch);
                }
            }
        }
    }
}