class TFWeaponBase : Weapon
{
	int TFWepFlags;

	flagdef Reloadable : TFWepFlags, 0;
	flagdef CanKick : TFWepFlags, 1;
	
	int ammoGiveReserve;
	int reloadSpeed;
	class<Ammo> ammoReserveType;
	Inventory ammoReserve;

	property AmmoGiveReserve : ammoGiveReserve;
	property ReloadSpeed : ReloadSpeed;
	property AmmoReserveType : ammoReserveType;

	Actor prevOwner;
	Actor owners[8];
	
	Default
	{
		Weapon.AmmoGive 0;
		
		+WEAPON.NOALERT
		+WEAPON.AMMO_OPTIONAL;
		+TFWeaponBase.Reloadable;
		+TFWeaponBase.CanKick;
		+FLOORCLIP
	}

	override void AttachToOwner(Actor other)
	{
		super.AttachToOwner(other);

		ammoReserve = FindInventory(AmmoReserveType);

		if (ammoReserve)
			A_GiveInventory(AmmoReserveType);
	}

	override void OnDrop(Actor dropper)
	{
		prevOwner = dropper;

		super.OnDrop(dropper);
	}

	override bool TryPickup(in out Actor toucher)
	{
		bool repeatPickup;
		for (int i = 0; i < 8; i++)
		{
			if (owners[i] == toucher)
			{
				repeatPickup = true;
				break;
			}
		}
		
		if (ShouldStay() && repeatPickup)
			return false;
		
		bool ammoAdded;
		if (!prevOwner)
			ammoAdded |= toucher.A_GiveInventory(AmmoReserveType, AmmoGiveReserve);

		if (ammoAdded)
			GoAwayAndDie();

		bool res = ammoAdded | super.TryPickup(toucher);

		if (res)
			owners[PlayerPawn(toucher).PlayerNumber()] = toucher;
		
		return res;
	}

	action void A_TFWeaponReady(int flags = 0)
	{
		if (CanReload())
			flags |= WRF_ALLOWRELOAD;
		else
			flags &=~ WRF_ALLOWRELOAD;

		if (invoker.bCanKick)
			flags |= WRF_ALLOWUSER1;
		else
			flags &=~ WRF_ALLOWUSER1;
		
		A_WeaponReady(flags);
	}
	
	action void SetWeaponState(StateLabel state, int layer = PSP_WEAPON)
	{
		let psp = Player.GetPSprite(layer);
		
		if (psp)
		{
			psp.SetState(invoker.FindState(state));
		}
	}

	action void A_ReloadPrimary(bool instant = true)
	{
		int amt = invoker.Ammo1.MaxAmount - invoker.Ammo1.Amount;
		
		if (instant)
		{
			for (int i = amt; i > 0 && invoker.ammoReserve.Amount; i--)
			{
				invoker.ammoReserve.Amount--;
				invoker.Ammo1.Amount++;
			}
		}
		else
		{
			invoker.ammoReserve.Amount--;
			invoker.Ammo1.Amount++;
		}
	}

	action bool CanReload()
	{
		return invoker.bReloadable && invoker.Ammo1.Amount != invoker.Ammo1.MaxAmount && invoker.ammoReserve && invoker.ammoReserve.Amount;
	}

	action void A_TFJumpIfNoAmmo(StateLabel st)
	{
		if (!CountInv(invoker.AmmoReserveType))
			SetWeaponState(st);
	}
	
	States
	{
		Spawn:
		Deselect:
		Select:
		Ready:
		Fire:
		Reload:
			#### AAAAAA 1 A_WeaponOffset(0, 12, WOF_ADD);
			#### A 1
			{
				A_SetTics(invoker.ReloadSpeed);
				A_ReloadPrimary();
			}
			#### AAAAAA 1 A_WeaponOffset(0, -12, WOF_ADD);
			#### # 0
			{
				SetWeaponState("Ready");
				return;
			}
		AltFire:
		Kick:
			#### ### 2 A_WeaponOffset(0, 40, WOF_ADD);
			#### # 2 A_StartSound("FIST/SWING", CHAN_WEAPON, CHANF_OVERLAP);
			#### # 2 A_Overlay(-1, "KickOverlay");
			#### # 2
			{
				A_CustomPunch((Clamp(vel.Length(), 4, 8)) * 4, true, 0, "KickPuff", 74);
				Radius_Quake(1, 12, 0, 1, 0);
			}
			#### #### 2;
			#### ### 2 A_WeaponOffset(0, -30, WOF_ADD);
			#### # 2
			{
				SetWeaponState("Ready");
				return;
			}
		KickOverlay:
			KLEG A 2 A_OverlayOffset(-1, 0, -120, WOF_ADD);
			KLEG ABBBA 1;
			TNT1 AA 2;
			Stop;
	}
}

////Puffs////

class TFMeleePuff : BulletPuff
{
	Default
	{
		Damagetype "Punch";
		ProjectileKickback 200;
		Species "fren";
		VSpeed 0.5;
		Scale 0.1;

		AttackSound "Metal/punch";

		RenderStyle "Translucent";
		Alpha 0.5;

		+NOBLOCKMAP
		+NOGRAVITY
		+ALLOWPARTICLES
		+RANDOMIZE
		+NOEXTREMEDEATH
		+THRUSPECIES
		+MTHRUSPECIES
		-BLOODSPLATTER
	}
	
	States
	{
		Spawn:
			SMOK ABCDEFGHIJKLMNOPQ 1;
		Melee:
			SMOK A 1 A_StartSound(AttackSound);
			SMOK BCDEFGHIJKLMNOPQ 1;
			Stop;
	}
}

class TFBulletPuff : BulletPuff Replaces BulletPuff
{	
	Default
	{
		Species "fren";
		Radius 1;
		Height 1;
		Scale 0.15;

		RenderStyle "Add";
		Alpha 0.75;
		
		+NOGRAVITY
		+NOBLOCKMAP
		+FLOORCLIP
		+NOEXTREMEDEATH
		+THRUSPECIES
		+MTHRUSPECIES
	}
	
	States
	{
		Spawn:
			BPUF A 1 NoDelay Bright
			{
				A_StartSound("puff/ric");
				A_SpawnDebris("BulletSpark");
				A_SpawnItemEx("GunSmoke");
			}
			BPUF BCD 1 Bright {	alpha -= 0.25; }
			Stop;
	}
}

class KickPuff : TFMeleePuff
{
	Default
	{
		DamageType "Kick";
		ProjectileKickback 600;
		Species "fren";
		AttackSound "Prison/punch";

		+PUFFONACTORS
		+BLOODLESSIMPACT
	}
}