class Fly : Actor 
{
	Vector3 spawnPos;

	Default
	{
		//$Category "NPC/Parasites"

		Species "Enemy";
		Monster;

		Health 1;
		Radius 3;
		Height 3;
		Speed 3;
		Mass 0;
		Scale 0.20;

		BloodColor "Yellow";

		+FLOORCLIP
		+CANPASS
		+NOINFIGHTING
		+DONTHARMCLASS
		+QUICKTORETALIATE
		+THRUACTORS
		+BOUNCEONWALLS
		+FLOAT
		+NOGRAVITY
		-COUNTKILL
	}

	override void PostBeginPlay()
	{
		super.PostBeginPlay();

		spawnPos = pos;
	}

	override void Tick()
	{
		super.Tick();

		if (pos.x > spawnPos.x + 64 || pos.x < spawnPos.x - 64)
		{
			vel.x = -vel.x;
		}
		if (pos.y > spawnPos.y + 64 || pos.y < spawnPos.y - 64)
		{
			vel.y = -vel.y;
		}
		if (pos.z > spawnPos.z + 64 || pos.z < spawnPos.z - 64)
		{
			vel.z = -vel.z;
		}
	}

	override void Die(Actor source, Actor inflictor, int dmgflags, Name MeansOfDeath)
	{
		A_StopSound(CHAN_BODY);

		super.Die(source, inflictor, dmgflags, MeansOfDeath);
	}

	States
	{
		Spawn:
			TNT1 A 0 NoDelay
			{
				A_StartSound("Flying/Fly", CHAN_BODY, CHANF_LOOPING);
				vel =
				(
					RandomPick[Fly](-Speed, -Speed + 1, Speed - 1, Speed),
					RandomPick[Fly](-Speed, -Speed + 1, Speed - 1, Speed),
					RandomPick[Fly](-Speed, -Speed + 1, Speed - 1, Speed)
				);
			}
		Idle:
			SFLY AABBCC 1;
			TNT1 A 0
			{
				A_Look();
				
				speed = RandomPick[Fly](1, 3, 3, 3, 5);
			}
			Loop;
		See:
			SFLY AABBCC 1;
			TNT1 A 0
			{
				if (target && Distance3DSquared(target) <= 24336)
				{
					SetStateLabel("Follow");
				}
				
				speed = RandomPick[Fly](1, 3, 3, 3, 5);
			}
			Loop;
		Follow:
			SFLY AABBCCAABBCC 1 A_Chase();
			TNT1 A 0
			{
				if (!target || (target && Distance3DSquared(target) > 24336))
				{
					SetStateLabel("Idle");
				}
			}
			Loop;
		XDeath:
		Death:
			TNT1 A 1;
			Stop;
	}
}

class SwarmFlySpawner : Actor
{
	Array<Actor> flies;
	
	Default
	{
		Health 1;
		Radius 32;
		Height 32;

		BloodColor "Yellow";
	}

	override void PostBeginPlay()
	{
		super.PostBeginPlay();

		for (int i = 0; i < 4; i++)
		{
			let f = Spawn("SwarmFly", pos + (Random[Fly](-radius, radius), Random[Fly](-radius, radius), Random[Fly](0, height)));
			SwarmFly(f).spawnPos = pos;
			SwarmFly(f).range = radius;

			flies.Push(f);
		}
	}

	States
	{
		Death:
			TNT1 A 1
			{
				for (int i = 0; i < flies.Size(); i++)
				{
					flies[i].A_Die();
				}
			}
			Stop;
	}
}

class SwarmFly : FXBase_Movement
{
	int range;
	Vector3 spawnPos;
	
	Default
	{
		//$Category "NPC/Parasites"

		Radius 3;
		Height 3;
		Speed 5;
		Scale 0.20;
	}

	override void Tick()
	{		
		super.Tick();

		if (pos.x > spawnPos.x + range || pos.x < spawnPos.x - range)
		{
			vel.x = -vel.x;
		}
		if (pos.y > spawnPos.y + range || pos.y < spawnPos.y - range)
		{
			vel.y = -vel.y;
		}
		if (pos.z > spawnPos.z + range || pos.z < spawnPos.z - range)
		{
			vel.z = -vel.z;
		}
	}

	override void Die(Actor source, Actor inflictor, int dmgflags, Name MeansOfDeath)
	{
		A_StopSound(CHAN_BODY);

		super.Die(source, inflictor, dmgflags, MeansOfDeath);
	}

	States
	{
		Spawn:
			TNT1 A 0 NoDelay A_StartSound("SFX/Buzz", CHAN_BODY, CHANF_LOOPING, pitch: FRandom[Fly](0.9, 1.1), startTime: FRandom[Fly](0.0, 0.5));
		Idle:
			SFLY ABCB 2
			{
				vel = (Random[Fly](-Speed, Speed), Random[Fly](-Speed, Speed), Random[Fly](-Speed, Speed));
			}
			Loop;
		XDeath:
		Death:
			TNT1 A 1;
			Stop;
	}
}